/*
 File:		FirePdu.java
 CVS Info:	$Id: FirePdu.java,v 1.2 1998/01/27 18:44:15 mcgredo Exp $
 Compiler:	jdk 1.1.6 
 */

package mil.navy.nps.dis;                                   // package to which we belong

import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;                                         // utility stuff we need
import java.io.*;                                           // input/output for serialization

/**
 *@version 1.0
 *@author <a href="mailto:mcgredo@stl.nps.navy.mil"> Don McGregor</a> (<a href="http://www.stl.nps.navy.mil/~mcgredo">http://www.stl.nps.navy.mil/~mcgredo</a>)
 *
 *<dt><b>Location:</b>
 *<dd><a href="http://www.stl.nps.navy.mil/dis-java-vrml/mil/navy/nps/dis/FirePdu.java">
 *  http://www.stl.nps.navy.mil/dis-java-vrml/mil/navy/nps/dis/FirePdu.java</a>
 *
 *<dt><b>Hierarchy Diagram:</b>
 *<dd><a href="images/PduClassHierarchy.jpg"><IMG SRC="images/PduClassHierarchyButton.jpg" ALIGN=ABSCENTER></a>
 *
 *<dt><b>Summary:</b>
 *<dd>The firing of a weapon shall be communicated by issuing a Fire PDU.
 *
 *<dt><b>Explanation:</b>
 *<dd>The fire pdu denotes the firing of a weapon. It inherits the header
 *  information from ProtocolDataUnit, an abstract class that contains
 *  assorted protocol information. It implements the IDs of what's
 *  firing, what's being shot at, the munition, the event, and where
 *  the munition is headed.<P>
 *
 *  As with other PDUs, it knows how to serialize and deserialize itself
 *  from the wire. It also knows how to clone itself, and knows how to
 *  calculate its size when sent to the wire.<P>
 *
 *<dt><b>History:</b>
 *<dd>		11Dec96	/Don McGregor		/New
 *<dd>		10Mar97	/Don McGregor		/changes for javadoc compliance
 *<dd>		16Apr97	/Don McGregor		/PrintStream passed to printValues
 *<dd>		8Dec97	/Ronan Fauglas		/changes for documentation templates + complements in documentation
 *<dd>		11Dec97	/Ronan Fauglas		/changed access methods: thisVariable() --> getThisVariable()
 *<dd>		14jan97	/Ronan Fauglas		/changed position to locationInWorldCoordinate
 *
 *<dt><b>References:</b>
 *<dd>		DIS Data Dictionary :<a href="http://stds.sc.ist.ucf.edu/dis/dis-dd/pdu/7c.htm">
 *		Fire PDU</a>	
 *<dd>		DIS-Java-VRML Working Group: <a href="http://www.stl.nps.navy.mil/dis-java-vrml/"> 
 *		http://www.stl.nps.navy.mil/dis-java-vrml/</a>
 *<dd>		DIS specification : IEEE 1278.1, Section 5.4.4.1, 4.4.3.2
 *
 *<dt><b>Note:</b>
 *<dd>   No accessor methods to nested records is provided.
 *
 *@see ProtocolDataUnit
 *@see PduElement 
 *@see SerializationInterface
 *@see DetonationPdu
 */
public class FirePdu extends ProtocolDataUnit 
{

    /**
     *Firing Entity Identification - This field shall identify the firing entity.
     */
    protected EntityID          firingEntityID;             // ID of entity that's doing the shooting

    /**
     *Target Entity Identification - This field shall identify the intended target.
     *
     *<dl>
     *<dt><b>Value:</b>
     *<dd>If the intended target is unknown, this field shall contain TARGET_ID_ UNKNOWN.
     *</dl>
     */
    protected EntityID          targetEntityID;             // ID of entity that is undergoing darwinian selection

    /**
     *Munition Identification - This field shall specify the entity ID of the fired munition 
     *if tracking data is required.
     *
     *<dl>
     *<dt><b>Value:</b>
     *<dd>A munition ID shall have a value of MUNITION _NOT_TRACKED if tracking data for the munition
     * is not required.
     *</dl>
     */
    protected EntityID          munitionID;                 // ID of munition being shot

    /**
     *Event Identification - This field shall contain an identification generated by the firing entity
     *to associate related firing and detonation events.
     */
    protected EventID           eventID;                    // ID of event

    /**
     *Fire Mission Index - This field shall identify the fire mission.
     *
     *<dl>
     *<dt><b>Value:</b>
     *<dd>If the fire mission is unknown this field shall contain NO_FIRE_MISSION.
     *</dl>
     */
    protected UnsignedInt       fireMissionIndex;           // ID of fire mission

    /**
     *Location in World Coordinate - This field shall specify the location, in world coordinates, from which the munition was launched.
     */
    protected WorldCoordinate     locationInWorldCoordinate;                   // x,y,z location, in 64 bit coords
    
    /**
     *Burst Descriptor - This field shall describe the type of munition fired, the warhead, the fuse,
     *the quantity, and the rate.
     */
    protected BurstDescriptor   burstDescriptor;           // info about burst

    /**
     *Velocity - This field shall specify the velocity of the fired munition at the point 
     *when the issuing simulation application intends the externally visible effects 
     *of the launch (e.g. exhaust plume or muzzle blast) to first become apparent.
     */
    protected LinearVelocity          velocity;                   // x, y, z velocity (3 floats)

    /**
     *This field shall specify the range that an entity's fire control system
     *has assumed in computing the fire control solution.
     *     
     *<dl>
     *<dt><b>Value:</b>
     *<dd> This field shall be represented by a 32-bit floating point number in meters. 
     *For systems where range is unknown or unavailable, this field shall contain a value of zero.
     *<dt><b>References:</b>
     *<dd>	DIS Data Dictionary: <a href="http://stds.sc.ist.ucf.edu/dis/dis-dd/pdu/83.htm">
     *		Range Field</a>
     *</dl>
     */
    protected float             range;                      // Assumed range (from shooter point of view)

    /**
     *Constant value--size of Fire PDU with header. Here:
     *<code>sizeOf = 96 bytes</code>
     */
    public final static int     sizeOf = 96;                // size of object as written to wire

/**
 *Default constructor--fills with zeros for all values.
 */
public FirePdu()
{
    super.setPduType(PDUTypeField.FIRE);

    firingEntityID  = new EntityID();
    targetEntityID  = new EntityID();
    munitionID      = new EntityID();
    eventID         = new EventID();

    fireMissionIndex = new UnsignedInt(0);

    locationInWorldCoordinate        = new WorldCoordinate(0, 0, 0);

    burstDescriptor = new BurstDescriptor();

    velocity        = new LinearVelocity(0, 0, 0);
    range           = 0;

    return;
}


public Object clone()
{
    // make a copy of the object. This requires a deep copy, so we don't have two
    // objects sharing pointers to the same data.

 FirePdu    newFirePdu = new FirePdu();

 newFirePdu.setFiringEntityID(this.getFiringEntityID());
 newFirePdu.setTargetEntityID(this.getTargetEntityID());
 newFirePdu.setMunitionID(this.getMunitionID());
 newFirePdu.setEventID(this.getEventID());
 newFirePdu.setFireMissionIndex(this.getFireMissionIndex());

 newFirePdu.setLocationInWorldCoordinate(this.getLocationInWorldCoordinate());
 
 newFirePdu.setBurstDescriptor(this.getBurstDescriptor());

 newFirePdu.setVelocity(this.getVelocity());
 newFirePdu.setRange(this.getRange());

 return newFirePdu;
}

/**
 *@exception RuntimeException when IO error occurs.
 */
public void serialize(DataOutputStream outputStream)
{
    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    super.serialize(outputStream);      // write out header info
    
    try
     {
        firingEntityID.serialize(outputStream);
        targetEntityID.serialize(outputStream);
        munitionID.serialize(outputStream);
        eventID.serialize(outputStream);
        fireMissionIndex.serialize(outputStream);

        locationInWorldCoordinate.serialize(outputStream);

        burstDescriptor.serialize(outputStream);
        velocity.serialize(outputStream);

    
        outputStream.writeFloat(range);
     }
    catch (IOException ioError)
    {
        throw new 
            RuntimeException("Exception in FirePdu. Error writing to wire.");
    }

    return;
}

/**
 *@exception RuntimeException when IO error occurs.
 */
public void deSerialize(DataInputStream inputStream)
{
    // order is important here, since we need to read in the same order as
    // specified by the DIS standard.
    
    super.deSerialize(inputStream);     // read in all the header info

    try
    {
        firingEntityID.deSerialize(inputStream);
        targetEntityID.deSerialize(inputStream);
        munitionID.deSerialize(inputStream);
        eventID.deSerialize(inputStream);
        fireMissionIndex.deSerialize(inputStream);

        locationInWorldCoordinate.deSerialize(inputStream);

        burstDescriptor.deSerialize(inputStream);
        velocity.deSerialize(inputStream);

        range = inputStream.readFloat();
     }
    catch (IOException ioError)
        {
            throw new 
                RuntimeException("Exception in FirePdu. Error reading from wire.");
        }

}

public int length()
{
    return sizeOf;          // EntityTypes are this long, always.
}

public String PDUName()
{
  return new String("Fire PDU");
}

public void printValues(int indentLevel, PrintStream printStream)
{
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;     // level of indentation for our superclass

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("Fire PDU-");

    // ugly wart: get the superclass over to the left a couple pixels, if we have any to spare,
    // so the header info will be indented a bit less.

    if(superclassIndent > 0)
      superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    firingEntityID.printValues(indentLevel, printStream);
    targetEntityID.printValues(indentLevel, printStream);
    munitionID.printValues(indentLevel, printStream);
    eventID.printValues(indentLevel, printStream);

    printStream.println(indent + "fireMissionIndex: "    + fireMissionIndex.longValue());

    locationInWorldCoordinate.printValues(indentLevel, printStream);

    burstDescriptor.printValues(indentLevel, printStream);
    velocity.printValues(indentLevel, printStream);

    printStream.println(indent + "range: "   + range);

    return;
}

    //Accessor methods

public EntityID getFiringEntityID()
{   return (EntityID)firingEntityID.clone();
}
public void setFiringEntityID(EntityID pFiringEntityID)
{firingEntityID = pFiringEntityID;
}

public EntityID getTargetEntityID()
{   return (EntityID)targetEntityID.clone();
}
public void setTargetEntityID(EntityID pTargetEntityID)
{targetEntityID = pTargetEntityID;
}

public EntityID getMunitionID()
{   return (EntityID)munitionID.clone();
}
public void setMunitionID(EntityID pMunitionID)
{munitionID = pMunitionID;
}

public EventID getEventID()
{   return (EventID)eventID.clone();
}
public void setEventID(EventID pEventID)
{eventID = pEventID;
}

public UnsignedInt getFireMissionIndex()
{   return (UnsignedInt)fireMissionIndex.clone();
}
public void setFireMissionIndex(UnsignedInt pFireMissionIndex)
{fireMissionIndex = pFireMissionIndex;
}
public void setFireMissionIndex(int pFireMissionIndex)
{ fireMissionIndex = new UnsignedInt(pFireMissionIndex);
}

public WorldCoordinate getLocationInWorldCoordinate()
{   return (WorldCoordinate)locationInWorldCoordinate.clone();
}
public void setLocationInWorldCoordinate(WorldCoordinate pWorldCoordinate)
{locationInWorldCoordinate = pWorldCoordinate;
}
public void setLocationInWorldCoordinate(double pX, double pY, double pZ)
{ locationInWorldCoordinate = new WorldCoordinate(pX, pY, pZ);
}

public BurstDescriptor getBurstDescriptor()
{   return (BurstDescriptor)burstDescriptor.clone();
}
public void setBurstDescriptor(BurstDescriptor pBurstDescriptor)
{burstDescriptor = pBurstDescriptor;
}


public LinearVelocity getVelocity()
{   return (LinearVelocity)velocity.clone();
}
public void setVelocity(LinearVelocity pVelocity)
{velocity = pVelocity;
}
public void setVelocity(float pX, float pY, float pZ)
{ velocity = new LinearVelocity(pX, pY, pZ);
}

public float getRange()
{   return range;
}
public void setRange(float pRange)
{range = pRange;
}

} // end of class FirePdu
